--[[ Text substitutions ]]
local L = MacroTalk_Locals

local subsBase = {
	["%%[Nn]"] = function() return UnitName("player") end,
	["%%[Zz]"] = GetRealZoneText,
	["%%[Ss][Zz]"] = GetSubZoneText,
	["%%[Ll][Oo][Cc]"] = function()
		local x, y = GetPlayerMapPosition("player")
		return ("%.0f, %.0f"):format(x * 100, y * 100)
	end,
	
	--[[
	Until an interface is in place, you can add straight text substitutions
	to this table. See the example below. Remove the -- from the front to use
	it. Note also that it's case sensitive unless you write both upper- and
	lower-case letters inside brackets like the substitutions above.]]
	
	--["%%tsinfo"] = "TeamSpeak info: Server: host.domain.com, Password: 12345",
}

local sexes = {
	L["<no %s>"]:format(L["gender"]),
	MALE:lower(),
	FEMALE:lower()
}

local unitInfoUnits = {
	[""] = "player",
	["[Tt]"] = "target",
	["[Ff]"] = "focus",
	["[Mm]"] = "mouseover",
	["[Pp]"] = "pet",
	["[Tt][Tt]"] = "targettarget",
}

local function ActualValue(unit)
	return UnitInParty(unit) or UnitInRaid(unit) or 
		(MacroTalk_InBattleground() and 
			UnitFactionGroup(unit) == UnitFactionGroup("player"))
end

local unitInfoSuffixes = {
	[""] = UnitName,
	
	["[Ll]"] = function(unit)
		local level = UnitLevel(unit)
		return level > 0 and level or level < 0 and "??"
	end,
	
	["[Cc]"] = UnitClass,
	
	["[Gg]"] = function(unit)
		return sexes[UnitSex(unit)]
	end,
	
	["[Gg][Bb]"] = function(unit)
		local sex = UnitSex(unit)
		return sex > 1 and sexes[UnitSex(unit)] or ""
	end,
	
	["[Rr]"] = function(unit)
		local race
		if UnitIsPlayer(unit) then
			race = UnitRace(unit)
		else
			race = UnitCreatureType(unit)
		end
		return race or L["<no %s>"]:format(RACE:lower())
	end,
	
	["[Rr][Bb]"] = function(unit)
		local race
		if UnitIsPlayer(unit) then
			race = UnitRace(unit)
		else
			race = UnitCreatureType(unit)
		end
		return race or ""
	end,
	
	["[Gg][Uu]"] = function(unit)
		local guild
		guild = GetGuildInfo(unit)
		return guild or "<no guild>"
	end,
	
	["[Gg][Uu][Bb]"] = function(unit)
		local guild
		guild = GetGuildInfo(unit)
		return guild or ""
	end,
	
	["[Rr][Mm]"] = function(unit)
		local nam, realm
		name, realm = UnitName(unit)
		return realm or GetRealmName()
	end,
		
	
	["[Hh]"] = function(unit)
		if ActualValue(unit) then
			return UnitHealth(unit).."/"..UnitHealthMax(unit)
		else
			return ("%.0f%%%%"):format(UnitHealth(unit))
		end
	end,
	
	["[Hh][Pp]"] = function(unit)
		return ("%.0f%%%%"):format(UnitHealth(unit)/UnitHealthMax(unit)*100)
	end,
	
	["[Mm][Aa]"] = function(unit)
		local max = UnitManaMax(unit)
		if max > 0 then
			return UnitMana(unit).."/"..UnitManaMax(unit)
		else
			return L["<no %s>"]:format(MANA:lower())
		end
	end,
	
	["[Mm][Aa][Bb]"] = function(unit)
		local max = UnitManaMax(unit)
		if max > 0 then
			return UnitMana(unit).."/"..UnitManaMax(unit)
		else
			return ""
		end
	end,
	
	["[Mm][Pp]"] = function(unit)
		local max = UnitManaMax(unit)
		if max > 0 then
			return ("%.0f%%%%"):format(max > 0 and UnitMana(unit)/max*100 or 0)
		else
			return L["<no %s>"]:format(MANA:lower())
		end
	end,
	
	["[Mm][Pp][Bb]"] = function(unit)
		local max = UnitManaMax(unit)
		if max > 0 then
			return ("%.0f%%%%"):format(max > 0 and UnitMana(unit)/max*100 or 0)
		else
			return ""
		end
	end,
	
	["[Ii][Cc]"] = function(unit)
		local index = GetRaidTargetIndex(unit)
		return index and "{"..(_G["RAID_TARGET_"..index]):lower().."}" or 
			L["<no %s>"]:format(EMBLEM_SYMBOL:lower())
	end,
	
	["[Ii][Cc][Bb]"] = function(unit)
		local index = GetRaidTargetIndex(unit)
		return index and "{"..(_G["RAID_TARGET_"..index]):lower().."}" or ""
	end,
}

for initial, unit in pairs(unitInfoUnits) do
	local noUnit = L["<no %s>"]:format(unit)
	for suffix, func in pairs(unitInfoSuffixes) do
		if initial ~= "" or suffix ~= "" then
			local code = "%%"..initial..suffix
			if subsBase[code] then
				error(L["MacroTalk: Conflicting substitutions: "..code])
			end
			subsBase[code] = function()
				return UnitExists(unit) and func(unit) or noUnit
			end
		end
	end
end

local substitutions = {}
local i = 1
for k, v in pairs(subsBase) do
	substitutions[i] = { code = k, func = v }
	i = i + 1
end

sort(substitutions, function(subs1, subs2)
	return subs2.code:len() < subs1.code:len()
end)

local OrigSendChatMessage = SendChatMessage
function SendChatMessage(text, ...)
	for _, substitution in ipairs(substitutions) do
		local func = substitution.func
		text = text:gsub(substitution.code,
			type(func) == "function" and func() or func)
	end
	return OrigSendChatMessage(text, ...)
end
